var Gedcom = {
	/** @protected Version level */
	Version : "1.0.2",

	/** @protected Internal parameters */
	Params : {
		/** Paths */
		path : {
			ressource : "ged/db2/",
			gedfile : "ged/db2/TGC55CLF.ged",
			archive : 'gedcom.tar.gz'
		},

		/** Elements ids */
		id : {
			canvas : "canvas",
			history : "history",
			detail : "detail",
			content : "content",
			log : "log",
			quick : "quick",
			quick_select : "quickSelect",
			close_detail : "closeDetail",
			download : "download"
		},

		/** Name of the query parameter for navigation */
		query_param : "root",

		/** display preferences */
		display : {
			download : true,
			log : true,
			private_record : false,
			geoloc : true,
			quickValue : true
		},

		/** Gedcom format indicator */
		indicator : {
			personne : "INDI",
			famille : "FAM",
			source : "SOUR",
			note : "NOTE",
			newLine : "CONT",
			append : "CONC"

		},

		/** Display log level */
		logLevel : 3

	},

	/** @protected IHM Management */
	IHM : {

		/** @public Initialize the IHM, hide/show elements according to the parameters and set texts */
		initialize : function() {
			$(Gedcom.Params.id.detail).hide();

			if (!Gedcom.Params.display.log) {
				$(Gedcom.Params.id.log).hide();
			}

			if (!Gedcom.Params.display.quickValue) {
				$(Gedcom.Params.id.quick).hide();
			}

			if (!Gedcom.Params.display.download) {
				$(Gedcom.Params.id.download).hide();
			} else {
				$(Gedcom.Params.id.download).firstDescendant().writeAttribute("src", Gedcom.Params.path.archive);
				$(Gedcom.Params.id.download).firstDescendant().update(Gedcom.Language.$.index.download);
			}

			$(Gedcom.Params.id.close_detail).update(Gedcom.Language.$.index.close);
			$(Gedcom.Params.id.history).firstDescendant().update(Gedcom.Language.$.index.history);
			$(Gedcom.Params.id.log).firstDescendant().update(Gedcom.Language.$.index.log);
		},

		/** @public Fill the values of the quick navigation menu */
		fillQuickValues : function(options) {
			Gedcom.Logger.debug("Fill quickSelect items");
			var select = $(Gedcom.Params.id.quick_select);

			options = [{
						text : "--- " + Gedcom.Language.$.index.quickSelect + " ---",
						value : ""
					}].concat(options);

			options.each(function(option) {
						select.appendChild(new Element("option").update(option.text)
								.writeAttribute("value", option.value));
					}, this);
		}
	},

	/** @protected Language management */
	Language : {
		/** @public current language definition */
		$ : null,

		/** @private current language */
		language : null,
		/** @private language defintions */
		defs : {},

		/** @public add a new language */
		addLanguage : function(key, def) {
			this.defs[key] = def;
			if (null == this.language) {
				this.setLanguage(key);
			}
		},

		/** @public define the current language */
		setLanguage : function(language) {
			this.language = language;
			this.$ = this.defs[language];
		}
	},

	/** @public GEDCOM file parser */
	Parser : {

		/** @private data extracted */
		DATAS : null,

		/**
		 * Retrieve a stored data
		 *
		 * @public
		 * @param {String}
		 *          id : data identifier
		 * @return {Gedcom.Entities} object stored or null is undefined
		 */
		getData : function(id) {
			return this.DATAS.get(id);
		},

		/**
		 * Load a GEDCOM file
		 *
		 * @public
		 * @param {String}
		 *          file : path of the file (local or web link)
		 */
		load : function(file) {
			Gedcom.Logger.info();
			Gedcom.Logger.info("Parser::Loading " + file);

			var id = Gedcom.Utils.id('iframe');
			Gedcom.Logger.debug("Adding iframe " + id);

			var iframe = new Element("iframe");
			iframe.writeAttribute("id", id);
			iframe.writeAttribute("style", "display:none;");
			iframe.writeAttribute("src", file);
			iframe.writeAttribute("onload", "Gedcom.Parser.loadCallback('" + id + "')");
			document.body.appendChild(iframe);
		},

		/**
		 * Callback of the iframe load
		 *
		 * @private
		 * @param {String}
		 *          id : iframe id
		 */
		loadCallback : function(id) {
			Gedcom.Logger.debug("Iframe " + id + " loaded");
			var o = $(id);

			Gedcom.Logger.debug("Reading iframe " + id);
			var ctn = o.contentWindow.document.body;
			if (ctn.getElementsByTagName('pre').length > 0) {
				ctn = ctn.childNodes[0];
			}

			Gedcom.Logger.debug("Removing iframe " + id);
			o.remove();

			this.parse(ctn.innerHTML);
		},

		/**
		 * Create an internal data node
		 *
		 * @private
		 * @param {String}
		 *          type : node type
		 * @param {String}
		 *          value : node value
		 * @param {Int}
		 *          level : node level
		 * @return {Object}
		 */
		node : function(type, value, level) {
			return {
				value : value,
				type : type,
				level : level,
				childs : []
			};
		},

		/**
		 * Parse the content of a GEDCOM file
		 *
		 * @private
		 * @param {String}
		 *          gedcom : file content
		 */
		parse : function(gedcom) {
			var lines = gedcom.split(/\r\n|\n|\r/g);
			Gedcom.Logger.debug(lines.length + " lines extracted");

			// Create root node
			var data = this.node(null, null);

			// Initialise parents
			var parents = [data];

			// Previous node level
			var _level = null;

			$A(lines).each(function(line, i) {
				var current = this.extract(line);

				// Erreur de traitement de la ligne
				if (null == current) {
					return;
				}

				// Réduction de la liste des parents
				if (null != _level && current.level <= _level) {
					$R(current.level, _level).each(function() {
								parents.pop();
							}, this);
				}

				// Recherche du dernier parent
				var parent = parents[parents.length - 1];

				if (current.ident == Gedcom.Params.indicator.append
						|| current.ident == Gedcom.Params.indicator.newLine) {
					var node = parent;
					var key = (node.level == 0) ? "text" : "value";

					var v = current.value || "";
					if (Object.isUndefined(node[key])) {
						node[key] = v;
					} else {
						node[key] += (current.ident == Gedcom.Params.indicator.newLine ? "<br/>" : "") + v;
					}

				} else {
					// Nouveau noeud
					var node = this.node(current.ident, current.value, current.level);

					// Ajout du nouveau noeud au parent
					parent.childs.push(node);
				}

				// Ajout du dernier noeud en tant que parent
				parents.push(node);

				// Mémorisation du niveau du noeud
				_level = current.level;
			}, this);

			// analyze
			this.analyze(data);
		},

		/**
		 * Create an internal data node
		 *
		 * @private
		 * @param {String}
		 *          type : node type
		 * @param {String}
		 *          value : node value
		 * @param {Int}
		 *          level : node level
		 * @return {Object}
		 */
		node : function(type, value, level) {
			return {
				value : value,
				type : type,
				level : level,
				childs : []
			};
		},

		/**
		 * Extract the line data of a GEDCOM file
		 *
		 * @private
		 * @param {String}
		 *          line
		 * @return {Object} data
		 */
		extract : function(line) {
			var result = line.match(/(\d) ([^ ]*) ?(.*)?/);
			// Ligne non conforme
			if (null == result) {
				return null;
			}
			result.shift();

			result = result.compact();
			return {
				level : result[0],
				ident : result[1],
				value : result[2]
			};
		},

		/**
		 * Analyze the extracted data
		 *
		 * @private
		 * @param {Array}
		 *          data
		 */
		analyze : function(data) {
			Gedcom.Logger.info();
			Gedcom.Logger.info("Parser::Extraction :");

			this.DATAS = $H();
			(data.childs || []).each(function(child, i) {
						try {

							var data = null;

							if (Gedcom.Params.indicator.personne == child.value) {
								data = new Gedcom.Entities.Personne(child);
							} else if (Gedcom.Params.indicator.famille == child.value) {
								data = new Gedcom.Entities.Famille(child);
							} else if (Gedcom.Params.indicator.source == child.value) {
								data = new Gedcom.Entities.Source(child);
							} else if (Gedcom.Params.indicator.note == child.value) {
								data = new Gedcom.Entities.Note(child);
							} else {
								Gedcom.Logger.warn("Parser::analyze -> unknow -> " + child.type
										+ (child.value ? ' (' + child.value + ')' : ''));
							}

							if (data) {
								this.DATAS.set(child.type, data);
							}

						} catch (e) {
							Gedcom.Logger.error("Parser::analyze -> each", e);
						}
					}, this);

			Gedcom.Logger.debug(this.DATAS.size() + " bloc chargés");

			// Post traitement
			var personnes = [];
			[Gedcom.Params.indicator.personne, Gedcom.Params.indicator.famille].each(function(type) {
						try {
							Gedcom.Logger.debug("Parser::analyze -> inject for " + type);
							this.DATAS.each(function(pair) {
										if (pair.value.type != type) {
											return;
										}
										try {
											if (Object.isFunction(pair.value.inject)) {
												pair.value.inject(this);
											}
											if (pair.value.type == Gedcom.Params.indicator.personne && pair.value.isPublic()) {
												personnes.push(pair.value);
											}
										} catch (e) {
											Gedcom.Logger.error("Parser::analyze -> inject", e);
										}
									}, this);
						} catch (e) {
							Gedcom.Logger.error("Parser::inject -> each", e);
						}
					}, this);

			// Liste des quickValues
			if (Gedcom.Params.display.quickValue) {
				var options = personnes.sortBy(function(p) {
							return p.getValue("name")[0];
						}, this).collect(function(p) {
							return {
								text : p.getDisplayName(Gedcom.Language.$.viewer.multipleNameSeparator, false),
								value : p.id
							};
						}, this);

				Gedcom.IHM.fillQuickValues(options);
			}

			Gedcom.Utils.show();
		}

	},

	/** @public GEDCOM file display */
	Viewer : {

		Params : {
			boxSpacerWidth : 20,
			boxSpacerHeight : 40,
			boxRadius : 10,
			boxBorderWidth : 2,
			boxMinWidth : 100,
			boxMinHeight : 20,

			sandBoxX : -500,
			sandBoxY : -500,

			margin : 10,
			textMargin : 5,

			arrow : true,

			font : "Arial",

			linkColor : "#EEEEEE"
		},

		// Variables privées
		/** @private * */
		familleParentDim : null,
		/** @private * */
		parentDim : null,
		/** @private * */
		childsDim : null,
		/** @private * */
		canvas : null,
		/** @private * */
		history : null,
		/** @private * */
		detail : null,
		/** @private * */
		content : null,
		/** @private * */
		r : null,
		/** @private * */
		root : null,
		/** @private * */
		textSandbox : null,

		/**
		 * Constructeur
		 *
		 * @param id
		 *          identifier de l'element HTML où effectuer le rendu
		 */
		initialize : function() {
			this.canvas = $(Gedcom.Params.id.canvas);
			this.history = $(Gedcom.Params.id.history);
			this.detail = $(Gedcom.Params.id.detail);
			this.content = $(Gedcom.Params.id.content);
			this.history.hide();
			this.detail.hide();

			Gedcom.Plugins.setRequired("familles");
			Gedcom.Plugins.setRequired("familleParent");
			Gedcom.Plugins.setRequired("sex");
		},

		/**
		 * Positionne le noeud racide de l'arbre à afficher.
		 *
		 * @param {Personne /
		 *          Famille} : racine de type 'Personne' ou 'Famille'.
		 */
		setRoot : function(root) {
			Gedcom.Logger.info();
			Gedcom.Logger.info("View::Définition de la racine");

			var newRoot = root;

			// Recherche de la famille à afficher
			if (Gedcom.Params.indicator.personne == root.type) {
				if (root.getValue("familles").length > 0) {
					newRoot = root.getValue("familles")[0];
				} else if (root.getValue("familleParent")) {
					newRoot = root.getValue("familleParent");
				}
				this.history.appendChild(new Element("div").update(this.createHtmlLink(root)));
				this.history.show();
			}

			// Mise à jour de la racine
			if (this.root && (this.root.id == newRoot.id)) {
				return;
			} else {
				this.root = newRoot;
				location.hash = this.root.id;
			}

			// Clean
			if (this.canvas.hasChildNodes()) {
				while (this.canvas.childNodes.length >= 1) {
					this.canvas.removeChild(this.canvas.firstChild);
				}
			}

			// Création du canvas
			this.r = Raphael(this.canvas, this.width, this.height);
			this.textSandbox = this.r.text(this.Params.sandBoxX, this.Params.sandBoxY);

			// Début du traitement
			try {
				this.analyze();
				this.display();
				// this.addBorder();
			} catch (e) {
				Gedcom.Logger.error("Viewer::setRoot", e);
			}
			Gedcom.Logger.info("View::Fin d'affichage");
		},

		/**
		 * Analyse les données.
		 */
		analyze : function() {
			Gedcom.Logger.info("View::Analyse du noeud racine");

			var familleParent = [];
			var parents = [];
			var childs = [];

			var familleParentDimOffset = 0;

			// Si le Père existe
			if (this.root.husb) {
				// Création de la boite pour le père
				parents.push(this.addBox(this.root.husb, "P"));

				if (null != this.root.husb.getValue("familleParent")) {
					var gp = this.root.husb.getValue("familleParent").husb;
					if (gp) {
						familleParent.push(this.addBox(gp, "GP"));
					}

					var gm = this.root.husb.getValue("familleParent").wife;
					if (gm) {
						familleParent.push(this.addBox(gm, "GP"));
					}
				} else {
					familleParentDimOffset += this.getBlocDimensions(this.root.husb).width + this.Params.boxSpacerWidth;
				}
			}

			if (this.root.wife) {
				parents.push(this.addBox(this.root.wife, "P"));

				if (null != this.root.wife.getValue("familleParent")) {
					var gp = this.root.wife.getValue("familleParent").husb;
					if (gp) {
						familleParent.push(this.addBox(gp, "GP"));
					}

					var gm = this.root.wife.getValue("familleParent").wife;
					if (gm) {
						familleParent.push(this.addBox(gm, "GP"));
					}
				}
			}

			for (var ic = 0; ic < this.root.childs.length; ic++) {
				childs.push(this.addBox(this.root.childs[ic], "C"));
			}

			// Calcul des Largeur / Hauteur
			{
				this.parentDim = this.getBlocDimensions(parents);
				this.parentDim.width += (parents.length - 1) * this.Params.boxSpacerWidth;

				this.familleParentDim = this.getBlocDimensions(familleParent);
				this.familleParentDim.width += familleParentDimOffset + (familleParent.length - 1)
						* this.Params.boxSpacerWidth;

				this.childsDim = this.getBlocDimensions(childs);
				this.childsDim.width += (childs.length - 1) * this.Params.boxSpacerWidth;
			}
			// FIN ------

			Gedcom.Logger.debug("View::Détail de la racine :");
			Gedcom.Logger.debug("&#160;>&#160;" + familleParent.length + " grand parents");
			Gedcom.Logger.debug("&#160;>&#160;" + parents.length + " parents");
			Gedcom.Logger.debug("&#160;>&#160;" + childs.length + " enfants");

			var height = this.familleParentDim.height + this.Params.boxSpacerHeight + this.parentDim.height
					+ this.Params.boxSpacerHeight + this.childsDim.height;
			var width = Math.max(this.familleParentDim.width, this.parentDim.width, this.childsDim.width);
			this.r.setSize(this.Params.margin * 2 + width, this.Params.margin * 2 + height);
		},

		/**
		 * Créé une boîte pour une personne dans la sandbox
		 *
		 * @param {Personne} :
		 *          la personne à afficher
		 * @return {Personne} personne
		 */
		addBox : function(p, level) {
			Gedcom.Logger.debug("Création de (" + level + ") " + this.createHtmlLink(p), 1);
			p.box = {};

			var attrs = {
				open : {
					"img" : Gedcom.Utils.getSexImage(p.getValue("sex")),
					"width" : 16,
					"height" : 16,
					"title" : Gedcom.Language.$.viewer.clickopen
				},
				tree : {
					"img" : null,
					"width" : 16,
					"height" : 16,
					"cursor" : "pointer",
					"title" : Gedcom.Language.$.viewer.clicknavigate
				},
				rect : {
					"stroke" : "#404040",
					"stroke-width" : this.Params.boxBorderWidth,
					"stroke-opacity" : 0.75,
					"fill" : "#CECECE",
					"fill-opacity" : 0.75,
					"cursor" : "pointer",
					"title" : Gedcom.Language.$.viewer.dbclicknavigate
				}
			};

			switch (p.getValue("sex")) {
				case "M" :
					attrs.rect.stroke = "#001E4C";
					attrs.rect.fill = "270-#B1C9ED-#698CB9";
					break;

				case "F" :
					attrs.rect.stroke = "#BF0000";
					attrs.rect.fill = "270-#F8C2E3-#EDB1B1";
					break;
			}

			if (p.isDead()) {
				attrs.rect['stroke'] = '#A0A0A0';
			}

			if ("GP" == level) {
				attrs.tree.img = "parent.png";
			}

			if ("C" == level && p.getValue("familles") && p.getValue("familles").length > 0) {
				attrs.tree.img = "childs.png";
			}

			var navigation = true;
			if (null == attrs.tree.img) {
				attrs.tree.height = 0;
				attrs.tree.width = 0;
				navigation = false;
			}

			if (!navigation) {
				delete attrs.rect.cursor;
				delete attrs.rect.title;
			}

			if (!p.isPublic()) {
				attrs.open.height = 0;
				attrs.open.width = 0;
				attrs.open.img = null;
			}

			var defaultTxtAttr = {
				"fill" : "#000",
				"font-size" : 14,
				"stroke-opacity" : 1,
				"text-anchor" : "middle",
				"cursor" : (navigation ? "pointer" : null),
				"title" : (navigation ? Gedcom.Language.$.viewer.dbclicknavigate : null)
			};

			var texts = p.getBoxText(defaultTxtAttr, navigation);
			var d = {
				width : 0,
				height : 0
			};
			texts.each(function(t) {
						if (t.txt) {
							var dim = this.getTextDimensions(t.txt, t.attrs);
							d.width = Math.max(d.width, dim.width);
							d.height += dim.height + this.Params.textMargin;
						}
					}, this);
			if (d.height > 0) {
				d.height -= this.Params.textMargin;
			}

			var w = Math.max(this.Params.boxMinWidth, d.width);
			var h = Math.max(this.Params.boxMinHeight, d.height);

			var imgs = ["open", "tree"];
			var imgH = 0;
			var imgW = 0;
			imgs.each(function(img) {
						var imgh = attrs[img].height;
						imgH += imgh + (imgh > 0 ? 2 : 0);
						imgW = Math.max(imgW, attrs[img].width);
					}, this);

			var x0 = this.Params.sandBoxX;
			var y0 = this.Params.sandBoxY;
			var W = w + this.Params.textMargin * 2 + imgW;
			var H = Math.max(imgH, h) + this.Params.textMargin * 2;

			p.box.rect = this.r.rect(x0, y0, W, H, this.Params.boxRadius).attr(attrs.rect);

			var imgOffsetY = 4;
			if (null != attrs.open.img) {
				p.box.open = this.r.image("resources/" + attrs.open.img, x0 + W - imgW - 2, y0 + imgOffsetY,
						attrs.open.width, attrs.open.height).attr({
							"title" : attrs.open.title,
							"cursor" : "pointer"
						});
				p.box.open.node.onclick = function() {
					this.openDetail(p);
				}.bind(this);
			}

			imgOffsetY += 2 + attrs.open.height;

			if (attrs.tree.height > 0) {
				p.box.tree = this.r.image("resources/" + attrs.tree.img, x0 + W - imgW - 2, y0 + imgOffsetY,
						attrs.tree.width, attrs.tree.height).attr(attrs.tree);

				p.box.tree.node.onclick = function() {
					this.setRoot(p);
				}.bind(this);

			}

			var textOffsetX = w / 2;
			var textOffsetY = 0;

			texts.each(function(t) {
						var dim = this.getTextDimensions(t.txt, t.attrs);
						if (t.txt) {
							p.box["txt_" + t.key] = this.r.text(textOffsetX + this.Params.textMargin + x0,
									this.Params.textMargin + y0 + dim.height / 2 + textOffsetY, t.txt, this.r.getFont(this.Params.font))
									.attr(t.attrs);
							textOffsetY += dim.height + this.Params.textMargin;
						}
					}, this);

			// Navigation
			if (navigation) {
				[p.box.rect].compact().each(function(o) {
							o.node.ondblclick = function() {
								this.setRoot(p);
							}.bind(this);
						}, this);
			}

			return p;
		},

		/**
		 * Affiche l'arbre
		 */
		display : function() {
			Gedcom.Logger.info("View::Affichage de l'arbre");
			try {

				var yOffset = 0;
				// Cas des parents avec parents
				{
					var xOffsetFamilleParent = (this.r.width - this.familleParentDim.width) / 2;
					var yOffsetFamilleParent = 0;
					var yOffsetParent = 0;
					[this.root.husb, this.root.wife].each(function(p) {
								if (p) {
									if (null != p.getValue("familleParent")) {

										// Positionnement du GP
										var gp;
										var o = p.getValue("familleParent").husb;
										if (o) {
											gp = this.placeBox(o, xOffsetFamilleParent, 0);
											xOffsetFamilleParent += this.getBlocDimensions(gp).width + this.Params.boxSpacerWidth;
											yOffsetFamilleParent = Math
													.max(yOffsetFamilleParent, this.getBlocDimensions(gp).height);
										}

										// Positionnement de la GM
										var gm;
										var o = p.getValue("familleParent").wife;
										if (o) {
											gm = this.placeBox(o, xOffsetFamilleParent, 0);
											xOffsetFamilleParent += this.getBlocDimensions(gm).width + this.Params.boxSpacerWidth;
											yOffsetFamilleParent = Math
													.max(yOffsetFamilleParent, this.getBlocDimensions(gm).height);
										}

										// Positionnement du Fils
										this.placeBox(p, this.getParentX(p), this.Params.boxSpacerHeight + this.familleParentDim.height);
										yOffsetParent = Math.max(yOffsetParent, this.getBlocDimensions(p).height);

										if (gp) {
											this.link(gp, p);
										}
										if (gm) {
											this.link(gm, p);
										}
									} else {
										xOffsetFamilleParent += this.getBlocDimensions(p).width + this.Params.boxSpacerWidth;
									}
								}
							}, this);
					if (yOffsetFamilleParent > 0) {
						yOffsetFamilleParent += this.Params.boxSpacerHeight;
					}
					yOffset += yOffsetFamilleParent;
				}

				// Cas des parents sans parents
				{
					if ((this.root.wife && null == this.root.wife.getValue("familleParent"))
							&& (this.root.husb && null == this.root.husb.getValue("familleParent"))) {

						var xOffsetParent = this.Params.margin;

						var p = this.placeBox(this.root.husb, xOffsetParent, yOffset);
						xOffsetParent += this.getBlocDimensions(p).width + this.Params.boxSpacerWidth;
						yOffsetParent = Math.max(yOffsetParent, this.getBlocDimensions(p).height);

						var m = this.placeBox(this.root.wife, xOffsetParent, yOffset);
						yOffsetParent = Math.max(yOffsetParent, this.getBlocDimensions(m).height);
					}
				}

				// Cas du mari sans parents
				{
					if ((this.root.husb && null == this.root.husb.getValue("familleParent"))
							&& (this.root.wife && null != this.root.wife.getValue("familleParent"))) {
						var offset = this.getBlocPosition(this.root.wife).x - this.Params.boxSpacerWidth
								- this.getBlocDimensions(this.root.husb).width;

						this.placeBox(this.root.husb, offset, this.getBlocPosition(this.root.wife).y - this.Params.margin);
					}
				}

				// Cas de la femme sans parents
				{
					if ((this.root.husb && null != this.root.husb.getValue("familleParent"))
							&& (this.root.wife && null == this.root.wife.getValue("familleParent"))) {

						var offset = this.getBlocPosition(this.root.husb).x
								+ this.getBlocDimensions(this.root.husb).width + this.Params.boxSpacerWidth;

						this.placeBox(this.root.wife, offset, this.getBlocPosition(this.root.husb).y - this.Params.margin);
					}
				}

				// Affichage du lien entre parents
				if (this.root.husb && this.root.wife) {
					this.link(this.root.husb, this.root.wife);
				}

				if (yOffsetParent > 0) {
					yOffsetParent += this.Params.boxSpacerHeight;
				}
				yOffset += yOffsetParent;

				// Affichage des enfants
				if (this.root.childs.length > 0) {

					// Positionnement des enfants
					var xOffsetChilds = this.Params.margin;
					for (var ic = 0; ic < this.root.childs.length; ic++) {
						var o = this.placeBox(this.root.childs[ic], xOffsetChilds, yOffset);
						xOffsetChilds += this.getBlocDimensions(o).width
								+ (ic < (this.root.childs.length - 1) ? this.Params.boxSpacerWidth : 0);
					}

					// Alignement avec les parents
					var px;
					if (this.root.husb && this.root.wife) {
						px = (this.getBlocPosition(this.root.husb).x + this.getBlocPosition(this.root.wife).x + this
								.getBlocDimensions(this.root.husb).width)
								/ 2;
					} else if (this.root.husb) {
						px = (this.getBlocPosition(this.root.husb).x + this.getBlocDimensions(this.root.husb).width) / 2;
					} else if (this.root.husb) {
						px = (this.getBlocPosition(this.root.wife).x + this.getBlocDimensions(this.root.wife).width) / 2;
					}

					var xTranslate = Math.max(0, (px - (this.Params.margin + xOffsetChilds) / 2));
					for (var ic = 0; ic < this.root.childs.length; ic++) {
						$H(this.root.childs[ic].box).each(function(pair) {
									pair.value.translate(xTranslate, 0);
								}, this);
					}

					// Ajout des lients
					for (var ic = 0; ic < this.root.childs.length; ic++) {
						this.link(this.root.childs[ic], [this.root.wife, this.root.husb]);
					}
				}

				// Vérification de la largeur
				var maxX = 0;
				var maxY = 0;
				if (this.root.husb && this.root.husb.getValue("familleParent")) {
					[this.root.husb.getValue("familleParent").husb, this.root.husb.getValue("familleParent").wife]
							.each(function(o) {
										if (o) {
											maxX = Math.max(maxX, this.getBlocPosition(o).x + this.getBlocDimensions(o).width);
											maxY = Math.max(maxY, this.getBlocPosition(o).y + this.getBlocDimensions(o).height);
										}
									}, this);
				}

				if (this.root.wife && this.root.wife.getValue("familleParent")) {
					[this.root.wife.getValue("familleParent").husb, this.root.wife.getValue("familleParent").wife]
							.each(function(o) {
										if (o) {
											maxX = Math.max(maxX, this.getBlocPosition(o).x + this.getBlocDimensions(o).width);
											maxY = Math.max(maxY, this.getBlocPosition(o).y + this.getBlocDimensions(o).height);
										}
									}, this);
				}

				if (this.root.husb) {
					maxX = Math.max(maxX, this.getBlocPosition(this.root.husb).x
									+ this.getBlocDimensions(this.root.husb).width);
					maxY = Math.max(maxY, this.getBlocPosition(this.root.husb).y
									+ this.getBlocDimensions(this.root.husb).height);
				}

				if (this.root.wife) {
					maxX = Math.max(maxX, this.getBlocPosition(this.root.wife).x
									+ this.getBlocDimensions(this.root.wife).width);
					maxY = Math.max(maxY, this.getBlocPosition(this.root.wife).y
									+ this.getBlocDimensions(this.root.wife).height);
				}

				if (this.root.childs.length > 0) {
					maxX = Math.max(maxX, this.getBlocPosition(this.root.childs[this.root.childs.length - 1]).x
									+ this.getBlocDimensions(this.root.childs[this.root.childs.length - 1]).width);
					$A(this.root.childs).each(function(c) {
								maxY = Math.max(maxY, this.getBlocPosition(c).y + this.getBlocDimensions(c).height);
							}, this);
				}
				maxX += this.Params.margin;
				maxY += this.Params.margin;

				this.r.setSize(maxX, maxY);
			} catch (e) {
				Gedcom.Logger.error("Viewer::display", e);
			}
		},

		/**
		 * Positionne le bloc d'une personne
		 *
		 * @param {Personne} :
		 *          personne à positionner
		 * @param {Long} :
		 *          offset en X par rapport à l'origine
		 * @param {Long} :
		 *          offset en Y par rapport à l'origine
		 * @return {Personne} p
		 */
		placeBox : function(p, oX, oY) {
			Gedcom.Logger.debug("&#160;>&#160;Positionnement de " + this.createHtmlLink(p));
			var hiddenOffsetX = -this.Params.sandBoxX;
			var hiddenOffsetY = -this.Params.sandBoxY;

			$H(p.box).each(function(pair) {
						pair.value.translate(hiddenOffsetX + oX, this.Params.margin + hiddenOffsetY + oY);
					}, this);

			return p;
		},

		/**
		 * Affiche le lien entre deux personnes
		 *
		 * @param {Personne} :
		 *          personne source
		 * @param {Personne,Personne[]} :
		 *          personne(s) destination.
		 * @return {Element} ligne tracée
		 */
		link : function(s, d) {
			Gedcom.Logger.debug("&#160;>&#160;Création d'un lien depuis " + this.createHtmlLink(s));
			var p = [];

			var attrs = {
				stroke : this.Params.linkColor,
				"stroke-width" : 2,
				"stroke-opacity" : 1,
				"arrow-end" : this.Params.arrow ? "classic" : "none",
				"arrow-start" : this.Params.arrow ? "classic" : "none"
			};

			if (Object.isArray(d)) {
				d = d.compact();
				if (1 == d.length) {
					d = d[0];
				}
			}

			if (!Object.isArray(d)) {
				// Same level
				if (this.getBlocPosition(s).y == this.getBlocPosition(d).y) {
					if (this.getBlocPosition(s).x > this.getBlocPosition(d).x) {
						var t = s;
						s = d;
						d = t;
					}

					// Recherche d

					attrs["stroke-width"] = 1;

					var ys = [0];
					if (this.root.marr) {
						ys.push(-(attrs['stroke-width'] + 2));
					}

					ys.each(function(offsetY) {
								p.push("M" + this.getBlocPosition(s).x + "," + (this.getBlocPosition(s).y + offsetY));
								p.push("m" + (this.Params.boxBorderWidth / 2) + ",0");
								p.push("m" + this.getBlocDimensions(s).width + ","
										+ (Math.min(this.getBlocDimensions(s).height, this.getBlocDimensions(d).height) / 2));
								p.push("l"
										+ (this.getBlocPosition(d).x - this.getBlocPosition(s).x
												- this.getBlocDimensions(s).width - this.Params.boxBorderWidth) + ",0");
							}, this);

					delete attrs['arrow-end'];
					delete attrs['arrow-start'];

					// Père fils
				} else {
					if (this.getBlocPosition(s).y > this.getBlocPosition(d).y) {
						var t = s;
						s = d;
						d = t;
					}
					p.push("M" + this.getBlocPosition(s).x + "," + this.getBlocPosition(s).y);
					p.push("m" + (this.getBlocDimensions(s).width / 2) + "," + this.getBlocDimensions(s).height);
					p.push("m0," + (this.Params.boxBorderWidth / 2));
					p.push("l0," + (this.Params.boxSpacerHeight / 2));
					p
							.push("l0,"
									+ (this.getBlocPosition(d).y - this.getBlocPosition(s).y - this.getBlocDimensions(s).height - this.Params.boxSpacerHeight));
					p.push("L" + ((this.getBlocPosition(d).x + this.getBlocDimensions(d).width / 2)) + ","
							+ (this.getBlocPosition(d).y - this.Params.boxSpacerHeight / 2 + (this.Params.boxBorderWidth / 2)));
					p.push("l0," + (this.Params.boxSpacerHeight / 2 - this.Params.boxBorderWidth / 2));

					delete attrs['arrow-start'];
				}
			} else {
				// Cas des parents
				if (this.getBlocPosition(d[0]).x > this.getBlocPosition(d[1]).x) {
					d.reverse();
				}
				var midx = this.getBlocPosition(d[1]).x
						- (this.getBlocPosition(d[1]).x - this.getBlocPosition(d[0]).x - this.getBlocDimensions(d[0]).width)
						/ 2;
				var midy = this.getBlocPosition(d[0]).y
						+ Math.min(this.getBlocDimensions(d[0]).height, this.getBlocDimensions(d[1]).height) / 2;

				p.push("M" + this.getBlocPosition(s).x + "," + this.getBlocPosition(s).y);
				p.push("m0,-" + (this.Params.boxBorderWidth / 2));
				p.push("m" + (this.getBlocDimensions(s).width / 2) + ",0");
				p.push("l0,-" + (this.Params.boxSpacerHeight / 2));
				p.push("L" + midx + ","
						+ (this.getBlocPosition(s).y - (this.Params.boxSpacerHeight / 2) - (this.Params.boxBorderWidth / 2)));
				p.push("L" + midx + "," + midy);

				delete attrs['arrow-end'];
			}

			if (0 == p.length) {
				return null;
			}

			return this.r.path(p.join("")).attr(attrs);
		},

		/**
		 * Ajoute une bordure à l'arbre
		 */
		addBorder : function() {
			Gedcom.Logger.debug("View::Ajout du cadre");
			if (false) {
				var step = 50;
				for (var i = step; i < this.r.width; i = i + step) {
					this.r.path(["M" + i + ",0", "l0," + this.r.height].join("")).attr({
								stroke : "#CECECE",
								"stroke-width" : 1,
								"stroke-opacity" : 0.5
							});
					this.r.text(i, 5, i, this.r.getFont(this.Params.font));
				}
				for (var i = step; i < this.r.height; i = i + step) {
					this.r.path(["M0," + i, "l" + this.r.width + ",0"].join("")).attr({
								stroke : "#CECECE",
								"stroke-width" : 1,
								"stroke-opacity" : 0.5
							});
					this.r.text(10, i, i, this.r.getFont(this.Params.font));
				}
			}

			this.r.rect(1, 1, this.r.width - 1, this.r.height - 1, 0).attr({
						stroke : "#000000",
						"stroke-width" : 1,
						"stroke-opacity" : 1
					});
		},

		/**
		 * Retourne les dimensions en largeur et hauteur d'un texte
		 *
		 * @param {String} :
		 *          texte à écrire
		 * @return {width:'', height:''}
		 */
		getTextDimensions : function(text, attr) {
			if (null == text) {
				return {
					width : 0,
					height : 0
				};
			}
			this.textSandbox.attr(Object.extend({
						'text-anchor' : 'start',
						'text' : text
					}, (attr || {})));
			return {
				width : this.textSandbox.getBBox().width,
				height : this.textSandbox.getBBox().height
			};
		},

		/**
		 * Calcule la place occupée par pour un groupe de personnes.
		 *
		 * @param {Personne /
		 *          Personne[]} : personne ou tableau de personne
		 * @return { width : '', height : ''}
		 */
		getBlocDimensions : function(a) {
			a = Object.isArray(a) ? a : [a];
			return {
				width : $A(a).sum(function(p) {
							return p.box.rect.getBBox().width;
						}, this),

				height : $A(a).max(function(p) {
							return p.box.rect.getBBox().height;
						})
			};
		},

		/**
		 * Retour la position d'une personne
		 *
		 * @param {Personne} :
		 *          personne
		 * @return {x:'', y:''}
		 */
		getBlocPosition : function(p) {
			return {
				x : p.box.rect.getBBox().x,
				y : p.box.rect.getBBox().y
			};
		},

		/**
		 * Calcule la position en X d'une personne par rapport à ses deux parents
		 *
		 * @param {Personne} :
		 *          personne
		 * @return {Long} position en x
		 */
		getParentX : function(p) {
			var gp = p.getValue("familleParent").husb;
			var gm = p.getValue("familleParent").wife;

			if (gp && gm) {
				// Aligment sur le milieu de l'espace entre les des deux parents
				return (this.getBlocPosition(gp).x + this.getBlocPosition(gm).x + this.getBlocDimensions(gp).width - this
						.getBlocDimensions(p).width)
						/ 2;
			} else if (gp) {
				// Alignement sur le grand père
				return (this.getBlocPosition(gp).x + (this.getBlocDimensions(gp).width - this.getBlocDimensions(p).width)
						/ 2);
			} else {
				// Alignement sur la grand mère
				return (this.getBlocPosition(gm).x + (this.getBlocDimensions(gm).width - this.getBlocDimensions(p).width)
						/ 2);
			}

			// Aligment sur le milieu de la largeur des deux parents
			return (this.getBlocPosition(gm).x + this.getBlocDimensions(gm).width - this.getBlocPosition(gp).x - this
					.getBlocDimensions(p).width)
					/ 2 + this.getBlocPosition(gp).x;
		},

		openDetail : function(p) {
			if (!p.isPublic()) {
				return;
			}

			Gedcom.Logger.debug("Affichage de " + this.createHtmlLink(p));

			p.loadPlugins(p.data, false);

			this.content.update("<table>" + Gedcom.Plugins.getPlugins().collect(function(plugin) {
						var v = plugin.writer(p, this);
						if (v) {
							return "<tr><th>" + plugin.name + "</th><td>" + v + "</td></tr>";
						} else {
							return "";
						}
					}, this).join("") + "</table>");

			this.detail.show();
		},

		createHtmlLink : function(p, f) {
			if (null == p) {
				return null;
			}
			return '<a href="javascript:void(Gedcom.Utils.show(\'' + (f ? f.id : p.id) + '\'));">'
					+ "<img src='resources/" + Gedcom.Utils.getSexImage(p.getValue("sex")) + "'/>" + " "
					+ p.getDisplayName(Gedcom.Language.$.viewer.multipleNameSeparator, true) + '</a>';
		}

	},

	/** @public Useful functions */
	Utils : {
		/** @private first index for the id generation */
		idSeed : 100,

		/**
		 * Generate a new id for a given prefix
		 *
		 * @public
		 * @param {String}
		 *          prefix
		 * @return {String} the generated id
		 */
		id : function(prefix) {
			return "Gedcom_" + (prefix || "") + "_" + ++this.idSeed;
		},

		/**
		 * Extract the first node matching the given type
		 *
		 * @public
		 * @param {Object}
		 *          n : parent node
		 * @param {String}
		 *          type : node identifier to search for
		 * @param {Boolean}
		 *          value : true to return the value else to return the object
		 * @return {Object} first matching node
		 */
		getChild : function(n, type, value) {
			if (null == n) {
				return null;
			}
			var v = (n.childs || []).find(function(c) {
						return c.type == type;
					});
			if (v && value) {
				v = v.value;
			}
			return v;
		},

		/**
		 * Extract all the nodes matching the given type
		 *
		 * @public
		 * @param {Object}
		 *          n : parent node
		 * @param {String}
		 *          type : node identifier to search for
		 * @param {Boolean}
		 *          value : true to return the value else to return the object
		 * @return {Array} nodes matching
		 */
		getChilds : function(n, type, value) {
			var res = [];
			(n.childs || []).each(function(c) {
						if (c.type == type) {
							if (value) {
								res.push(c.value);
							} else {
								res.push(c);
							}
						}
					});
			return res;
		},

		/**
		 * Extract a note containing a date and place structure
		 *
		 * @public
		 * @param {Object}
		 *          n : parent node
		 * @return {Object} node
		 */
		extractDateAndPlace : function(n) {
			if (null == n) {
				return null;
			}
			return {
				value : n.value,
				date : this.getChild(n, "DATE", true),
				place : this.getChild(n, "PLAC", true),
				address : this.getChild(n, "ADDR", true),
				age : this.getChild(n, "AGE", true),
				source : this.getChild(n, "SOUR", true),
				notes : this.getChilds(n, "NOTE", true)
			};
		},

		/**
		 * Get the image name corresponding to the person's sex
		 *
		 * @public
		 * @param {String}
		 *          sex
		 * @return {String} name of the image
		 */
		getSexImage : function(s) {
			switch (s) {
				case "M" :
					return "male.png";
				case "F" :
					return "female.png";
				default :
					return "unknown.png";
			}
		},

		/**
		 * Get the value of a query string parameter
		 *
		 * @public
		 * @param {String}
		 *          name of the queryString parameter
		 * @return {String} value of the parameter. Null value if the parameter is not found
		 */
		getQueryParameter : function(parameterName) {
			var queryString = window.top.location.search.substring(1);
			var parameterName = parameterName + "=";
			if (queryString.length > 0) {
				var begin = queryString.indexOf(parameterName);
				if (begin != -1) {
					begin += parameterName.length;
					var end = queryString.indexOf("&", begin);
					if (end == -1) {
						end = queryString.length;
					}
					return unescape(queryString.substring(begin, end));
				}
			}
			return null;
		},

		/**
		 * Display the address map
		 *
		 * @public
		 * @param {String}
		 *          element's id containing the address
		 */
		displayAddress : function(id) {
			var o = $(Gedcom.Plugins.Params.map.iframe + id);
			if (o.visible()) {
				o.hide();
				$(Gedcom.Plugins.Params.map.link + id)
						.update(Gedcom.Language.$.plugins.helper.dateAndPlace.displayMap);
			} else {
				if (o.src == "") {
					var q = $(id).innerHTML.replace(/<br\/?>/ig, " ");
					o.src = "http://maps.google.fr/maps?&q=" + q + "&output=embed";
				}
				o.show();
				$(Gedcom.Plugins.Params.map.link + id).update(Gedcom.Language.$.plugins.helper.dateAndPlace.hideMap);
			}
		},

		/**
		 * Display the family tree
		 *
		 * @private
		 * @param {String}
		 *          Person or family id to display.
		 */
		show : function(id) {
			var query = id || Gedcom.Utils.getQueryParameter(Gedcom.Params.query_param)
					|| Gedcom.Parser.DATAS.find(function(d) {
								return d.value.type == Gedcom.Params.indicator.famille;
							})[0];

			var p = Gedcom.Parser.getData(query);
			if (!p) {
				return;
			}
			Gedcom.Viewer.setRoot(p);

			switch (p.type) {
				case Gedcom.Params.indicator.personne :
					Gedcom.Viewer.openDetail(p);
					break;

				case Gedcom.Params.indicator.famille :
					$(Gedcom.Params.id.detail).hide();
					break;
			}
		}
	},

	/** @protected Internal entities */
	Entities : {
		Personne : Class.create({
					type : null,
					data : null,
					id : null,
					plugin : null,
					pluginsLoaded : false,

					getValue : function(key) {
						return this.plugin[key];
					},

					setValue : function(key, value) {
						this.plugin[key] = value;
					},

					inject : function(parser) {
						Gedcom.Plugins.getPlugins(true).each(function(plugin) {
									if (Object.isFunction(plugin.inject)) {
										plugin.inject(this, parser);
									}
								}, this);
					},

					initialize : function(data) {
						this.type = Gedcom.Params.indicator.personne;
						this.plugin = {};
						this.id = data.type;
						this.data = data;
						try {
							this.loadPlugins(data, true);
						} catch (e) {
							Gedcom.Logger.error("Parser::Personne::initialize -> id:" + this.id, e);
						}
					},

					loadPlugins : function(data, required) {
						if (!this.pluginsLoaded) {
							Gedcom.Plugins.getPlugins(required).each(function(plugin) {
										plugin.parser(this, data);
									}, this);
							if (!required) {
								this.pluginsLoaded = true;
							}
						}
					},

					isDead : function() {
						return !!this.getValue("deat");
					},

					isPublic : function() {
						if (Gedcom.Params.display.private_record) {
							return true;
						}
						return ("privacy" != this.getValue("privacy"));
					},

					getBoxText : function(defaultTxtAttr, navigation) {
						var lines = [];
						lines.push({
									key : "name",
									txt : this.getDisplayName("\n", true),
									attrs : Object.extend({
												"font-weight" : "bold"
											}, defaultTxtAttr)
								});
						lines.push({
									key : "dates",
									txt : this.getDisplayDate(),
									attrs : defaultTxtAttr
								});
						return lines;
					},

					getDisplayName : function(nameJoiner, includeSex) {
						try {
							var txt = [];
							if (this.isPublic()) {
								if (includeSex && this.getValue("sex")) {
									switch (this.getValue("sex")) {
										case "M" :
											txt.push(Gedcom.Language.$.personne.sex.M);
											break;

										case "F" :
											txt.push(Gedcom.Language.$.personne.sex.F);
											break;
									}
								}
								if (this.getValue("name").length > 0) {
									txt.push(this.getValue("name").join(nameJoiner));
								}
							} else {
								txt.push("- " + Gedcom.Language.$.personne.privacy + " -");
							}
							return txt.join(" ");
						} catch (e) {
							Gedcom.Logger.error("Parser::getDisplayName", e);
						}
					},

					getDisplayDate : function() {
						try {
							if (!this.isPublic()) {
								return null;
							}

							if (!this.getValue("birt") && !this.getValue("deat")) {
								return null;
							}

							return [this.getValue("birt") && this.getValue("birt").date || "",
									this.getValue("deat") && this.getValue("deat").date || ""].join(" - ");

						} catch (e) {
							Gedcom.Logger.error("Parser::getDisplayDate", e);
						}
					}
				}),
		Plugin : Class.create({
					key : null,
					name : null,
					order : 5000,
					active : true,

					initialize : function(data) {
						try {
							// Initialisation des données
							Object.extend(this, data);
							this.name = Gedcom.Language.$.plugins[this.key] && Gedcom.Language.$.plugins[this.key].title
									|| this.key;
							Gedcom.Plugins.addPlugin(this);
						} catch (e) {
							Gedcom.Logger.error("Plugin::initialize", e);
						}
					},

					parser : function(p, data) {
						try {
							if (Object.isFunction(this.doParser)) {
								this.doParser.call(p, this, data);
							}
						} catch (e) {
							Gedcom.Logger.error("Plugin::" + this.key + "::parser", e);
						}
					},

					writer : function(data, scope) {
						try {
							if (Object.isFunction(this.doWriter)) {
								return this.doWriter.call(this, data, scope);
							} else {
								return data.plugin[this.key];
							}
						} catch (e) {
							Gedcom.Logger.error("Plugin::" + this.key + "::writer", e);
						}
					},

					inject : function(p, parser) {
						try {
							if (Object.isFunction(this.doInject)) {
								this.doInject.call(p, this, parser);
							}
						} catch (e) {
							Gedcom.Logger.error("Plugin::" + this.key + "::inject", e);
						}
					}

				}),
		Famille : Class.create({
					type : null,
					id : null,
					husb : null,
					wife : null,
					marr : null,
					childs : null,

					initialize : function(data) {
						try {
							// Extraction des données
							{
								this.type = Gedcom.Params.indicator.famille;

								this.id = data.type;

								// Mari
								this.husb = Gedcom.Utils.getChild(data, "HUSB", true);

								// Femme
								this.wife = Gedcom.Utils.getChild(data, "WIFE", true);

								// Informations du mariage
								this.marr = Gedcom.Utils.extractDateAndPlace(Gedcom.Utils.getChild(data, "MARR"));

								// Enfants
								this.childs = Gedcom.Utils.getChilds(data, "CHIL", true);

							}

						} catch (e) {
							Gedcom.Logger.error("Parser::Famille::initialize -> id:" + this.id, e);
						}
					},

					inject : function(parser) {
						if (this.husb) {
							this.husb = parser.getData(this.husb);
						}
						if (this.wife) {
							this.wife = parser.getData(this.wife);
						}
						this.childs = this.childs.collect(function(child) {
									return parser.getData(child);
								}, this);
					}

				}),
		Source : Class.create({
					type : null,
					id : null,

					title : null,
					abbr : null,
					auth : null,
					publ : null,
					text : null,
					notes : null,

					initialize : function(data) {
						try {
							// Extraction des données
							{
								this.type = Gedcom.Params.indicator.source;

								this.id = data.type;

								this.title = Gedcom.Utils.getChild(data, "TITL", true);
								this.abbr = Gedcom.Utils.getChild(data, "ABBR", true);
								this.auth = Gedcom.Utils.getChild(data, "AUTH", true);
								this.publ = Gedcom.Utils.getChild(data, "PUBL", true);
								this.text = Gedcom.Utils.getChild(data, "TEXT", true);
								this.notes = Gedcom.Utils.getChilds(data, "NOTE", true);
							}

						} catch (e) {
							Gedcom.Logger.error("Parser::Source::initialize -> id:" + this.id, e);
						}
					}
				}),
		Note : Class.create({
					type : null,
					id : null,
					text : null,

					initialize : function(data) {
						try {
							// Extraction des données
							{
								this.type = Gedcom.Params.indicator.note;

								this.id = data.type;

								// Text
								this.text = data.text;
							}

						} catch (e) {
							Gedcom.Logger.error("Parser::Note::initialize -> id:" + this.id, e);
						}
					}
				}),

		/** @public initialization */
		initialize : function() {
			Gedcom.Logger.info("Entities::initialize -> add required plugins");
			["deat", "privacy", "sex", "name", "birt"].each(function(p) {
						Gedcom.Plugins.setRequired(p);
					}, this);
		}
	},

	/** @protected Plugins management */
	Plugins : {

		/** @private list of the loaded plugins entites */
		plugins : [],

		/** @private Store of the loaded plugins (Only contains identifier) */
		pluginsLoaded : $H(),

		/** @private Store of the required plugins (Only contains identifier) */
		pluginsRequired : $H(),

		/** @public initialization */
		initialize : function() {
		},

		/** @public internal parameters */
		Params : {
			map : {
				link : 'mapLink',
				iframe : 'map'
			}
		},

		/**
		 * Set a plugin as required
		 *
		 * @public
		 * @param {String}
		 *          plugin identifier
		 */
		setRequired : function(key) {
			this.pluginsRequired.set(key, true);
		},

		/**
		 * Add a new plugin
		 *
		 * @public
		 * @param {Gedcom.Entities.Plugin}
		 *          plugin
		 */
		addPlugin : function(plugin) {
			if (this.pluginsLoaded.get(plugin.key)) {
				Gedcom.Logger.warn("Plugins:: Plugin [" + plugin.key + "]" + plugin.name + " already loaded");
				return;
			}

			if (plugin.active) {
				this.plugins.push(plugin);
				this.pluginsLoaded.set(plugin.key, true);
			}
		},

		/**
		 * Retrieve the plugin list.
		 *
		 * @param {Boolean}
		 *          required (null for retrieving all loaded plugins)
		 * @return {Array} list of the matching plugins
		 */
		getPlugins : function(required) {
			return this.plugins.filter(function(plugin) {
				return (Object.isUndefined(required) || (required && this.pluginsRequired.get(plugin.key)) || (!required && !this.pluginsRequired
						.get(plugin.key)));
			}, this);
		},

		/** @public Sort plugin list by the order attribute */
		sort : function() {
			Gedcom.Logger.info("Plugins:: Tri des plugins");
			this.plugins = this.plugins.sortBy(function(plugin) {
						return plugin.order;
					});
		},

		/** @public check if all the required plugins are loaded */
		checkRequired : function() {
			Gedcom.Logger.info("Plugins:: Vérification des plugins requis");
			var error = false;
			this.pluginsRequired.keys().each(function(plugin) {
						if (!this.pluginsLoaded.get(plugin)) {
							Gedcom.Logger.error("Plugin::checkRequired => plugin " + plugin + " missing");
							error = true;
						}
					}, this);
			if (error) {
				throw "Plugins manquant";
			}
		},

		/**
		 * Extract a group of date and place from data
		 *
		 * @private
		 * @param {String}
		 *          key : indicator or the root node
		 * @param {Gedcom.Entities.Plugin}
		 *          me : current plugin
		 * @param {Object}
		 *          data : node
		 */
		doParserDateAndPlaceGroupHelper : function(key, me, data) {
			this.setValue(me.key, Gedcom.Utils.getChilds(data, key).collect(function(o) {
								return Gedcom.Utils.extractDateAndPlace(o);
							}, this));
		},

		/**
		 * Return the display value of a group of date and place
		 *
		 * @private
		 * @param {Gedcom.Entities.Personne}
		 *          data : the person object
		 * @param {Object}
		 *          scope
		 * @return {String} the value to display
		 */
		doWriterDateAndPlaceGroupHelper : function(data, scope) {
			var d = data.getValue(this.key);
			return d && d.length > 0 ? Gedcom.Plugins
					.displayData(d, Gedcom.Plugins.displayDateAndPlace.bind(scope)) : null;
		},

		/**
		 * Extract a date and place from data
		 *
		 * @private
		 * @param {String}
		 *          key : indicator or the root node
		 * @param {Gedcom.Entities.Plugin}
		 *          me : current plugin
		 * @param {Object}
		 *          data : node
		 */
		doParserDateAndPlaceHelper : function(key, me, data) {
			this.setValue(me.key, Gedcom.Utils.extractDateAndPlace(Gedcom.Utils.getChild(data, key)));
		},

		/**
		 * Return the display value of a date and place
		 *
		 * @private
		 * @param {Gedcom.Entities.Personne}
		 *          data : the person object
		 * @param {Object}
		 *          scope
		 * @return {String} the value to display
		 */
		doWriterDateAndPlaceHelper : function(data, scope) {
			return Gedcom.Plugins.displayDateAndPlace(data.getValue(this.key));
		},

		/**
		 * Return the display value of group of data
		 *
		 * @private
		 * @param {Gedcom.Entities.Personne}
		 *          data : the person object
		 * @param {Object}
		 *          scope
		 * @return {String} the value to display
		 */
		doWriterGroupHelper : function(data, scope) {
			var d = data.getValue(this.key);
			return d && d.length > 0 ? Gedcom.Plugins.displayData(d) : null;
		},

		/**
		 * Return the display value of a group of notes
		 *
		 * @private
		 * @param {Array}
		 *          notes
		 * @return {String} the value to display
		 */
		displayNotes : function(notes) {
			return Gedcom.Plugins.displayData(notes, function(n) {
						if (n.match(/@N[0-9]+@/)) {
							return Gedcom.Parser.getData(n).text;
						} else {
							return n;
						}
					});
		},

		/**
		 * Return the display value of a group of sources
		 *
		 * @private
		 * @param {Array}
		 *          sources
		 * @return {String} the value to display
		 */
		displaySources : function(sources) {
			return Gedcom.Plugins.displayData(sources, function(n) {

						if (n.match(/@.+@/)) {
							var o = Gedcom.Parser.getData(n);
							var s = [];
							if (o.title) {
								s.push("<b>" + Gedcom.Language.$.plugins.helper.source.title + " :</b> " + o.title);
							}
							if (o.abbr) {
								s.push("<b>" + Gedcom.Language.$.plugins.helper.source.titleAbbr + " :</b> " + o.abbr);
							}
							if (o.auth) {
								s.push("<b>" + Gedcom.Language.$.plugins.helper.source.author + " :</b> " + o.auth);
							}
							if (o.publ) {
								s.push("<b>" + Gedcom.Language.$.plugins.helper.source.dateAndPlace + " :</b> " + o.publ);
							}
							if (o.text) {
								s.push("<b>" + Gedcom.Language.$.plugins.helper.source.text + " :</b> " + o.text);
							}
							if (o.notes && o.notes.length > 0) {
								s.push("<b>" + Gedcom.Language.$.plugins.helper.source.notes + " :</b> <span class='childs'>"
										+ Gedcom.Plugins.displayNotes(o.notes) + "</span>");
							}
							return s.join("<br/>");
						} else {
							return n;
						}

					});
		},

		/**
		 * Return the display value of a place and date
		 *
		 * @private
		 * @param {Object}
		 *          n : data
		 * @return {String} the value to display
		 */
		displayDateAndPlace : function(n) {
			if (null == n) {
				return null;
			}

			var fn = function(n) {
				var shtml = [];
				if (n.value) {
					switch (n.value) {
						case "Y" :
							shtml.push(Gedcom.Language.$.common.yes);
							break;

						case "N" :
							shtml.push(Gedcom.Language.$.common.no);
							break;

						default :
							shtml.push(n.value);
					}
					shtml.push("<br/>");
				}
				if (n.date) {
					shtml.push(Gedcom.Language.$.common.when + " " + n.date);
				}
				if (n.age) {
					shtml.push("(" + Gedcom.Language.$.plugins.helper.dateAndPlace.age + " " + n.age + ")");
				}
				if (n.place) {
					shtml.push(Gedcom.Language.$.common.where + " " + Gedcom.Plugins.displayLocation(n.place, true));
				}
				if (n.address) {
					shtml.push("<div style='margin-top:5px;'><b>"
							+ Gedcom.Language.$.plugins.helper.dateAndPlace.address + " :</b><br/>"
							+ Gedcom.Plugins.displayLocation(n.address, false))
							+ "</div>";
				}
				if (n.source) {
					shtml.push("<div style='margin-top:5px;'><b>"
							+ Gedcom.Language.$.plugins.helper.dateAndPlace.source + " :</b>"
							+ Gedcom.Parser.getData(n.source).text)
							+ "</div>";
				}
				if (n.notes && n.notes.length > 0) {
					shtml.push("<div style='margin-top:5px;'><b>" + Gedcom.Language.$.plugins.helper.dateAndPlace.notes
							+ " :</b><div class='childs'>" + Gedcom.Plugins.displayNotes(n.notes) + "</div>" + "</div>");
				}
				return shtml.join(" ");
			}.bind(this);

			return Gedcom.Plugins.displayData(n, fn);
		},

		/**
		 * Return the display value of an address with the show map link
		 *
		 * @private
		 * @param {Object}
		 *          s : location
		 * @param {Boolean}
		 *          linkInline : display the map link inline
		 * @return {String} the value to display
		 */
		displayLocation : function(s, linkInline) {
			var id = Gedcom.Utils.id("address");
			var html = [];
			html.push("<span id='" + id + "'>" + s + "</span>");
			if (Gedcom.Params.display.geoloc) {
				html.push(linkInline ? " " : "<br/>");
				html.push('<a href="javascript:void(0);" onclick="Gedcom.Utils.displayAddress(\'' + id + '\');" id="'
						+ this.Params.map.link + id + '">' + Gedcom.Language.$.plugins.helper.dateAndPlace.displayMap
						+ '</a>');
				html.push('<iframe id="' + this.Params.map.iframe + id
						+ '" style="display:none;width:100%;height:400px;"></iframe>');
				html.push(linkInline ? " " : "<br/>");
			}
			return html.join("");
		},

		/**
		 * Return the display value of a group of data
		 *
		 * @private
		 * @param {Object/Array}
		 *          n : data to display
		 * @param {Function}
		 *          fn : function used to format the data. Value if returned when fn is not defined
		 * @return {}
		 */
		displayData : function(n, fn) {
			fn = fn ? fn : function(o) {
				return o;
			};
			if (Object.isArray(n)) {
				if (0 == n.length) {
					return null;
				}
				n = n.compact();
				if (n.length > 1) {
					return "<ul>" + $A(n).collect(function(o) {
								return "<li>" + fn(o) + "</li>";
							}, this).join("") + "</ul>";
				} else {
					return fn(n[0]);
				}
			} else {
				return fn(n);
			}
		}
	},

	/** @protected Internal Logger */
	Logger : {

		/** @private available log levels */
		logLevels : {
			DEBUG : 0,
			INFO : 1,
			WARN : 2,
			ERROR : 3
		},

		/**
		 * Display a debug information
		 *
		 * @public
		 * @param {String}
		 *          t : text to display
		 */
		debug : function(t) {
			if (Gedcom.Params.logLevel <= this.logLevels.DEBUG) {
				this.display(this.logLevels.DEBUG, t);
			}
		},

		/**
		 * Display an info information
		 *
		 * @public
		 * @param {String}
		 *          t : text to display
		 */
		info : function(t) {
			if (Gedcom.Params.logLevel <= this.logLevels.INFO) {
				this.display(this.logLevels.INFO, t);
			}
		},

		/**
		 * Display a warn information
		 *
		 * @public
		 * @param {String}
		 *          t : text to display
		 */
		warn : function(t) {
			if (Gedcom.Params.logLevel <= this.logLevels.WARN) {
				this.display(this.logLevels.WARN, t);
			}
		},

		/**
		 * Display an error information
		 *
		 * @public
		 * @param {String}
		 *          t : text to display
		 * @param {Object}
		 *          error
		 */
		error : function(t, error) {
			if (Gedcom.Params.logLevel <= this.logLevels.ERROR) {
				this.display(this.logLevels.ERROR, t);
			}
			console.log(error);
		},

		/**
		 * Display an information
		 *
		 * @private
		 * @param {level}
		 *          level : error level
		 * @param {String}
		 *          t : text to display
		 */
		display : function log(level, t) {
			if (!Gedcom.Params.display.log) {
				return;
			}

			var c = "&#160;";
			if (null != t) {

				// Calcul de la date;
				var d = new Date();
				var dts = [2, 2, 2, 3];
				var dt = [d.getHours(), d.getMinutes(), d.getSeconds(), d.getMilliseconds()].collect(function(v, i) {
							v = v.toString();
							return $R(v.length, dts[i], true).collect(function() {
										return "0";
									}).join("") + v;
						}, this).join(":");

				var lt;
				switch (level) {
					case this.logLevels.DEBUG :
						lt = "DEBUG";
						break;
					case this.logLevels.INFO :
						lt = "INFO";
						break;
					case this.logLevels.WARN :
						lt = "WARN";
						break;
					case this.logLevels.ERROR :
						lt = "ERROR";
						break;
				}

				//
				c = "[" + dt + "]&#160;[" + lt + "]&#160;" + t;
			}
			$(Gedcom.Params.id.log).appendChild(new Element("div").addClassName(lt).update(c));
		}
	},

	/** @public initialization */
	initialize : function() {
		this.Entities.initialize();
		this.IHM.initialize();
		this.Viewer.initialize();

		this.Plugins.checkRequired();
		this.Plugins.sort();

		this.Parser.load(this.Params.path.gedfile);

	}
};